/*
Copyright (c) 2020 Krzysztof Mitko

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
*/



#include <stdio.h>
#include <stdlib.h>
#include "debiensko.h"
#include "xlsxwriter/xlsxwriter.h"

void BrinePrintXlsx(lxw_worksheet *worksheet, int row, int col, brine_t brine) {
  worksheet_write_number(worksheet, row, col, brine.cl, NULL);
  worksheet_write_number(worksheet, row+1, col, brine.ca, NULL);
  worksheet_write_number(worksheet, row+2, col, brine.mg, NULL);
  worksheet_write_number(worksheet, row+3, col, brine.so, NULL);
  worksheet_write_number(worksheet, row+4, col, Na(brine), NULL);
  worksheet_write_number(worksheet, row+5, col, TDS(brine), NULL);
  worksheet_write_number(worksheet, row+6, col, density(brine), NULL);
  worksheet_write_number(worksheet, row+7, col, brine.v, NULL);
}

// Program starts here...
int main(int argc, char** argv) {
  brine_t feed; // Coal mine water
  brine_t ROpermeate;
  brine_t ROretentate;
  brine_t VCfeed;
  brine_t VCconcentrate;
  brine_t RCCfeed;
  rcc_results_t RCClyes;
  double recovery = 0;
  double P = 0;
  int col = 2;
  int row = 11;
  lxw_workbook  *workbook  = workbook_new("result.xlsx");
  lxw_worksheet *worksheet = workbook_add_worksheet(workbook, NULL);
  if (argc != 6) {
    printf("Feed chloride concentration [kg/m3] : ");
    scanf("%lg", &feed.cl);
    printf("Feed calcium concentration [kg/m3]  : ");
    scanf("%lg", &feed.ca);
    printf("Feed magnesium concentration [kg/m3]: ");
    scanf("%lg", &feed.mg);
    printf("Feed sulphate concentration [kg/m3] : ");
    scanf("%lg", &feed.so);
    printf("Feed flow rate [m3/h]               : ");
    scanf("%lg", &feed.v);
  } else {
    feed.cl = strtod(argv[1], NULL);
    feed.ca = strtod(argv[2], NULL);
    feed.mg = strtod(argv[3], NULL);
    feed.so = strtod(argv[4], NULL);
    feed.v = strtod(argv[5], NULL);
  }
  printf("-----------------------------------------------------\n");
  printf("Feed water\n");
  printf("-----------------------------------------------------\n");
  worksheet_write_string(worksheet, 2, 0, "Cl- [kg/m3]", NULL);
  worksheet_write_string(worksheet, 3, 0, "Ca2+ [kg/m3]", NULL);
  worksheet_write_string(worksheet, 4, 0, "Mg2+ [kg/m3]", NULL);
  worksheet_write_string(worksheet, 5, 0, "SO42- [kg/m3]", NULL);
  worksheet_write_string(worksheet, 6, 0, "Na+ [kg/m3]", NULL);
  worksheet_write_string(worksheet, 7, 0, "TDS [%]", NULL);
  worksheet_write_string(worksheet, 8, 0, "d [%]", NULL);
  worksheet_write_string(worksheet, 9, 0, "V [m3/h]", NULL);
  worksheet_write_string(worksheet, 0, 1, "Feed water", NULL);
  worksheet_write_string(worksheet, row, 0, "Energy consumption [kWh]", NULL);
  BrinePrintXlsx(worksheet, 2, 1, feed);
  BrinePrintf(feed);
  if (TDSmass(feed) > 45) {
    printf("\n-----------------------------------------------------\n");
    printf("Skipping RO entirely, feed too saline\n");
    printf("-----------------------------------------------------\n");
    VCfeed = feed;
  } else {
    col = 5;
    printf("\n-----------------------------------------------------\n");
    printf("Calculating the reverse osmosis\n");
    printf("-----------------------------------------------------\n");
    // Pressure required by the RO
    // It should not exceed the 70 bar
    do {
      recovery = recovery + 0.1;
      ROpermeate = DiluateRO(feed, recovery);
      ROretentate = ConcentrateRO(feed, ROpermeate);
      P = Pf(feed, ROretentate);
    // } while (P < 68);
    } while (ROretentate.cl < 36);
    printf("\nRequired RO pressure: %#.4g bar\n", P);
    printf("\nRO permeate\n");
    BrinePrintf(ROpermeate);
    printf("\nRO retentate\n");
    BrinePrintf(ROretentate);
    printf("\nEnergy consumption of RO: %#.4g kWh\n", ROpermeate.v * EnergyRO(recovery, P, 0));
    VCfeed = ROretentate;
    worksheet_write_string(worksheet, 0, 2, "Reverse osmosis", NULL);
    worksheet_write_string(worksheet, 1, 2, "Feed", NULL);
    worksheet_write_string(worksheet, 1, 3, "Permeate", NULL);
    worksheet_write_string(worksheet, 1, 4, "Retentate", NULL);
    BrinePrintXlsx(worksheet, 2, 2, feed);
    BrinePrintXlsx(worksheet, 2, 3, ROpermeate);
    BrinePrintXlsx(worksheet, 2, 4, ROretentate);
    worksheet_write_string(worksheet, row, 1, "Reverse osmosis", NULL);
    worksheet_write_number(worksheet, row, 2, ROpermeate.v * EnergyRO(recovery, P, 0), NULL);
    row++;
  }
  worksheet_write_string(worksheet, 11, 3, "Amount of distillate [kg/h]", NULL);
  worksheet_write_string(worksheet, 11, 4, "Evaporator", NULL);
  worksheet_write_string(worksheet, 12, 4, "Crystallizer", NULL);
  worksheet_write_string(worksheet, 11, 6, "Amount of solids [kg/h]", NULL);
  worksheet_write_string(worksheet, 11, 7, "Salt", NULL);
  worksheet_write_string(worksheet, 12, 7, "Gypsum", NULL);
  worksheet_write_string(worksheet, row, 1, "Evaporator", NULL);
  printf("\n-----------------------------------------------------\n");
  printf("Calculating the evaporator\n");
  printf("-----------------------------------------------------\n");
  printf("\nFeed water\n");
  BrinePrintf(VCfeed);
  worksheet_write_string(worksheet, 0, col, "Evaporator", NULL);
  worksheet_write_string(worksheet, 1, col, "Feed", NULL);
  BrinePrintXlsx(worksheet, 2, col, VCfeed);
  printf("\nConcentrate\n");
  VCconcentrate = ConcentrateVC(feed);
  RCCfeed = VCconcentrate;
  BrinePrintf(VCconcentrate);
  worksheet_write_string(worksheet, 1, col+1, "Concentrate", NULL);
  BrinePrintXlsx(worksheet, 2, col+1, VCconcentrate);
  printf("\nAmount of distillate produced: %#.4g m3/h \n", DiluateVC(VCfeed));
  worksheet_write_number(worksheet, 11, 5, DiluateVC(VCfeed), NULL);
  printf("\nEnergy consumption: %#.4g kWh\n", EnergyVC() * DiluateVC(VCfeed));
  worksheet_write_number(worksheet, row, 2, EnergyVC() * DiluateVC(VCfeed), NULL);
  worksheet_write_string(worksheet, row+1, 1, "Crystallizer", NULL);
  printf("\n-----------------------------------------------------\n");
  printf("Calculating the crystallizer\n");
  printf("-----------------------------------------------------\n");
  printf("\nFeed water\n");
  BrinePrintf(RCCfeed);
  worksheet_write_string(worksheet, 0, col+2, "Crysallizer", NULL);
  worksheet_write_string(worksheet, 1, col+2, "Feed", NULL);
  worksheet_write_string(worksheet, 1, col+3, "Post-crystallization lyes", NULL);
  BrinePrintXlsx(worksheet, 2, col+2, RCCfeed);
  printf("\nPost-crystallization lyes:\n");
  RCClyes = ConcentrateRCC(RCCfeed);
  BrinePrintXlsx(worksheet, 2, col+3, RCClyes.lyes);
  BrinePrintf(RCClyes.lyes);
  printf("\nAmount of distillate produced: %#.4g kg/h\n", RCClyes.water);
  worksheet_write_number(worksheet, 12, 5, RCClyes.water, NULL);
  printf("\nAmount of salt produced: %#.4g kg/h\n", RCClyes.salt);
  worksheet_write_number(worksheet, 11, 8, RCClyes.salt, NULL);
  printf("\nAmount of gypsum produced: %#.4g kg/h\n", RCClyes.gypsum);
  worksheet_write_number(worksheet, 12, 8, RCClyes.gypsum, NULL);
  printf("\nEnergy consumption: %#.4g kWh\n", EnergyRCC() * (RCCfeed.v - RCClyes.lyes.v));
  worksheet_write_number(worksheet, row+1, 2, EnergyRCC() * (RCCfeed.v - RCClyes.lyes.v), NULL);
  if (RCClyes.lyes.cl <= CHLORIDELIMIT || RCClyes.lyes.ca < 0 || RCClyes.lyes.mg < 0 || RCClyes.lyes.so < 0 || RCClyes.salt < 0 || RCClyes.water < 0 || RCClyes.gypsum < 0 || Na(RCClyes.lyes) < 0 ) {
    printf("\nWARNING: results are unreliable. Maybe the starting positions were incorrect, maybe the crystallization is not be possible in these conditions.\n");
    worksheet_write_string(worksheet, row+3, 0, "WARNING: results are unreliable. Maybe the starting positions were incorrect, maybe the crystallization is not be possible in these conditions", NULL);
    workbook_close(workbook);
    return 1;
  }
  workbook_close(workbook);
  return 0;
}
